@@ -1,6 +1,6 @@
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2018.01 ($Rev: 570 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: forcrsv, forc_cnst, copy_to_cpd_forc_and_forcp, 
 !             cnstrnt, rplcps, rbinuc, change_of_coordinate_system
@@ -31,12 +31,12 @@
 !  Consortium.
 !   The activity of development of this program set has been supervised by Takahisa Ohno.
 !
-! $Id: b_Ionic_System.f90 570 2017-04-21 20:34:50Z yamasaki $
+! $Id: b_Ionic_System.f90 633 2020-12-01 05:11:03Z jkoga $
 subroutine forcrsv(natm,ityp,imdtyp,iwei,amion,cpd_l,tkb,nrsv,imdalg &
      & ,frsv,iw_cnst)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2018.01 ($Rev: 570 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: forcrsv
 !
@@ -119,7 +119,7 @@ subroutine forc_cnst(natm,ityp,amion,cps,cnst_typ,nfcatm,ia_cnst,imdtyp &
      &, sgmc,gca)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2018.01 ($Rev: 570 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: forc_cnst
 !
@@ -197,7 +197,7 @@ end subroutine forc_cnst
 subroutine copy_to_cpd_forc_and_forcp(katm,natm,cpd_l,forc_l,cpd,forc,forcp)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2018.01 ($Rev: 570 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: copy_to_cpd_forc_and_forcp
 !
@@ -231,7 +231,7 @@ subroutine cnstrnt(natm,ityp,amion,cps,cnst_typ,nfcatm,ia_cnst,imdtyp &
      &, sgmc,sigma)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2018.01 ($Rev: 570 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: cnstrnt
 !
@@ -312,7 +312,7 @@ end subroutine cnstrnt
 subroutine rplcps(cps,ityp,mode,natm2,natm,iwei)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2018.01 ($Rev: 570 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: rplcps
 !
@@ -349,7 +349,7 @@ end subroutine rplcps
 subroutine rbinuc(pos,natm)
 !=======================================================================
 !
-!  PROGRAM  PHASE/0 2018.01 ($Rev: 570 $)
+!  PROGRAM  PHASE/0 2016.01 ($Rev: 633 $)
 !
 !  SUBROUINE: rbinuc
 !
@@ -549,6 +549,7 @@ subroutine gnrt_supercell_symm_operations(natm2,katm,natm,napt,kopr,nopr,op &
   logical :: isComplete
 
   integer,       dimension(natm2)                :: npfatm
+  integer :: icount
 
   allocate(iop_opxop(nopr,nopr));
   do iop = 1, nopr
@@ -594,18 +595,21 @@ subroutine gnrt_supercell_symm_operations(natm2,katm,natm,napt,kopr,nopr,op &
      
   nopr_supercell = nopr + ngen_tl
   call substitute_npfatm()
+  icount = 0
   EXTEND: do 
      if (nopr_supercell > natm*nopr) then
         write(6,*) 'WARNING: Force symmetrization might be a failure.'
         nopr_supercell = nopr
         exit EXTEND
      end if
-
+     icount = icount+1
+!     if(mype==0) write(0,*) 'nopr_sup, icount ',nopr_supercell,icount
      ! MAKE TABLES (REDUNDANT SPACE-GROUP)
      isComplete = .true.
      nopr_local_temp = nopr_supercell
 
      do iop = 1, nopr_local_temp
+!        if(mype==0) write(0,*) 'iop, nopr_local_temp',iop,nopr_local_temp
         do jop = 1, nopr_local_temp
            iop_temp = iop_opxop(iop_supercell(iop),iop_supercell(jop))
 
@@ -721,6 +725,198 @@ contains
 !-----------------------------------------------------------------------------------
 end subroutine gnrt_supercell_symm_operations
 
+subroutine gnrt_supercell_symm_operations_apt(natm2,katm,natm,napt,kopr,nopr,op &
+     &  , koprpaf,tau,ngen_tl,tau_tl,napt_tl,lattice_system,mode &
+     &  , dim2,  napt_supercell, iop_supercell,  nopr_supercell &
+     &  , dim2e, iwei)
+! ==== coded by T. Yamasaki after a provided code by Usami-san on Nov. 2013. April 2014 ===
+! 
+  use m_Const_Parameters,  only : DP, PAI2, DELTA10,DELTA07
+  use m_Parallelization,   only : mype
+  implicit none
+  integer,parameter :: modeTAU = 1, modeAPT = 2
+  integer, intent(in)                            :: natm2,katm,natm,kopr,nopr
+  integer, intent(in), dimension(katm,kopr)      :: napt
+  real(kind=DP),intent(in),dimension(3,3,kopr)   :: op
+  integer, intent(in)                            :: koprpaf
+  real(kind=DP),intent(in),dimension(3,koprpaf,2):: tau
+  integer, intent(in)                            :: ngen_tl
+  real(kind=DP),intent(in),dimension(3,ngen_tl,2):: tau_tl
+  integer, intent(in), dimension(katm,ngen_tl)   :: napt_tl
+  character(len=9), intent(in)                   :: lattice_system
+  integer, intent(in)                            :: mode
+  integer, intent(in)                            :: dim2
+  integer, intent(out),dimension(katm,dim2)      :: napt_supercell
+  integer, intent(out),dimension(dim2)           :: iop_supercell
+  integer, intent(out)                           :: nopr_supercell
+  integer, intent(in)                            :: dim2e
+!  integer, intent(out),dimension(kopr)           :: nope_supercell
+!  integer, intent(out),dimension(dim2e,kopr)     :: pope_supercell
+  integer, intent(in), dimension(katm)           :: iwei
+
+  integer, allocatable, dimension(:)             :: nope_supercell
+  integer, allocatable, dimension(:,:)           :: pope_supercell
+  
+  integer        :: ia, iop, jop, ip
+
+  integer,             allocatable, dimension(:,:)   :: iop_opxop   ! d(nopr:nopr)
+  real(kind=DP), dimension(3,3) :: op_temp
+  integer                       :: iop_temp, napt1_temp
+  real(kind=DP), dimension(3)   :: tau_temp
+  real(kind=DP) ::  delta
+  integer :: i, j, nopr_local_temp
+  logical :: isComplete
+
+  integer,       dimension(natm2)                :: npfatm
+  integer :: icount
+  allocate(nope_supercell(kopr))
+  allocate(pope_supercell(dim2e,kopr))
+  allocate(iop_opxop(nopr,nopr));
+  do iop = 1, nopr
+     do jop = 1, nopr
+        op_temp = matmul(op(:,:,jop), op(:,:,iop))
+        iop_opxop(iop,jop) = operatorExists_in_nop()
+     end do
+  end do
+
+  nope_supercell = 0
+  napt_supercell = 0
+
+  ! COPY
+  do iop = 1, nopr
+     iop_supercell(iop) = iop
+!-------------------
+     napt_supercell(:,iop) = napt(:,iop)
+     nope_supercell(iop) = 1
+     pope_supercell(1,iop) = iop
+  end do
+
+  if(ngen_tl >= 1) then
+     OP_LOOP: do iop = 1, nopr
+        delta = dabs(op(1,1,iop)-1.0d0) + dabs(op(2,1,iop)) +       dabs(op(3,1,iop)) &
+             &+ dabs(op(1,2,iop)) +       dabs(op(2,2,iop)-1.0d0) + dabs(op(3,2,iop)) &
+             &+ dabs(op(1,3,iop))       + dabs(op(2,3,iop)) +       dabs(op(3,3,iop)-1.0d0)
+        if(delta < DELTA10) exit OP_LOOP
+     end do OP_LOOP
+     if(iop <= nopr) then
+!!$        write(6,'(" # of A Unit operator = ",i8)') iop
+     else
+        stop ' no Unit operator <gnrt_supercell_symm_operations>'
+     end if
+     do i = 1, ngen_tl
+        iop_supercell(nopr+i) = iop
+        napt_supercell(:,nopr+i) = napt_tl(:,i)
+        nope_supercell(iop) = nope_supercell(iop)+1
+        pope_supercell(1+i,iop) = nopr+i
+     end do
+  end if
+     
+  nopr_supercell = nopr + ngen_tl
+  call substitute_npfatm()
+  icount = 0
+  EXTEND: do 
+     if (nopr_supercell > natm*nopr) then
+        write(6,*) 'WARNING: Force symmetrization might be a failure.'
+        nopr_supercell = nopr
+        exit EXTEND
+     end if
+     icount = icount+1
+     isComplete = .true.
+     nopr_local_temp = nopr_supercell
+
+     do iop = 1, nopr_local_temp
+        !if(mype==0) write(0,*) 'iop, nopr_local_temp',iop,nopr_local_temp
+        do jop = 1, nopr_local_temp
+           iop_temp = iop_opxop(iop_supercell(iop),iop_supercell(jop))
+           if(napt_supercell(1,iop)>0) &
+           &  napt1_temp = npfatm(napt_supercell(npfatm(napt_supercell(1,iop)),jop))
+           
+           if (.not. operatorExists(kopr,iop_temp,napt1_temp,dim2e, &
+                     natm2,npfatm,nope_supercell,pope_supercell)) then
+              nopr_supercell = nopr_supercell + 1
+              nope_supercell(iop_temp) = nope_supercell(iop_temp) + 1
+              pope_supercell(nope_supercell(iop_temp),iop_temp) = nopr_supercell
+              if (nopr_supercell > dim2) cycle EXTEND
+              iop_supercell(nopr_supercell) = iop_temp
+              do i = 1, natm
+                 if(napt_supercell(i,iop)>0)            &
+                 &  napt_supercell(i, nopr_supercell) = &
+                 &  npfatm(napt_supercell(npfatm(napt_supercell(i, iop)), jop))
+              end do
+              isComplete = .false.
+           end if
+        end do
+     end do
+     if (isComplete) exit EXTEND
+  end do EXTEND
+
+  deallocate(iop_opxop)
+  deallocate(nope_supercell)
+  deallocate(pope_supercell)
+!!$  mnope_supercell = maxval(nope_supercell(1:nopr))
+
+contains
+  subroutine substitute_npfatm()
+    integer :: nb, ia
+    nb = 0
+    do ia = 1, natm
+       if(iwei(ia) == 2) then
+          nb = nb + 1
+          npfatm(natm+nb) = ia
+       end if
+       npfatm(ia) = ia
+    end do
+  end subroutine substitute_npfatm
+
+  integer function operatorExists_in_nop()
+    real(kind=DP) :: delta
+    integer :: nope
+    real(kind=DP), dimension(3,3) :: opd
+
+    do nope = 1, nopr
+       opd = op_temp - op(:,:,nope)
+       delta = opd(1,1)*opd(1,1) + opd(1,2)*opd(1,2) + opd(1,3)*opd(1,3) &
+            &+ opd(2,1)*opd(2,1) + opd(2,2)*opd(2,2) + opd(2,3)*opd(2,3) &
+            &+ opd(3,1)*opd(3,1) + opd(3,2)*opd(3,2) + opd(3,3)*opd(3,3)
+       if(delta < 1.0d-3) then
+          operatorExists_in_nop = nope
+          return
+       end if
+    end do
+    operatorExists_in_nop = -1
+    if(operatorExists_in_nop == -1) then
+       write(6,'(" iop, jop = ",2i8)') iop,jop
+       write(6,'(" !Illegal value of iop_opxop(iop,jop) = ",i8)') iop_opxop(iop,jop)
+       call flush(6)
+       stop '!Illegal value of iop_opxop(iop,jop)'
+    end if
+  end function operatorExists_in_nop
+!-----------------------------------------------------------------------------------
+
+  logical function operatorExists(kopr,iop_temp,napt1_temp,dim2e,natm2,npfatm,nope_supercell,pope_supercell)
+    ! return .true. if the operator (op_temp, tau_temp) is found in the operator list op_local.
+    integer, intent(in):: kopr,iop_temp,napt1_temp,dim2e,natm2
+    integer, intent(in), dimension(natm2) :: npfatm
+    integer, intent(in), dimension(kopr)  :: nope_supercell
+    integer, intent(in), dimension(dim2e,kopr) :: pope_supercell
+    real(kind=DP) :: delta
+    real(kind=DP), dimension(3) :: temp_a, temp
+    integer :: nope, nopei
+
+    do nopei = 1, nope_supercell(iop_temp)
+       nope = pope_supercell(nopei,iop_temp)
+       if(npfatm(napt_supercell(1,nope)) == napt1_temp) then
+          operatorExists = .true. ! found
+          return
+       end if
+    end do
+    operatorExists = .false.  ! not found
+    return
+  end function operatorExists
+!-----------------------------------------------------------------------------------
+end subroutine gnrt_supercell_symm_operations_apt
+
+
 subroutine fd_supercell_symmetrize(natm2,katm,natm,napt,nopr,op,kopr,iwei &
      & , f_l, f_wk, npfatm, iop_supercell)
 !
@@ -1258,6 +1454,292 @@ end subroutine fd_supercell_symmetrize
 !!$!-----------------------------------------------------------------------------------
 !!$end subroutine fd_symmetrize_sc2
 
+subroutine set_iatomn_default(elen,ntyp, speciesname, iatomn)
+  use m_Const_Parameters, only : DP
+  implicit none
+  integer, intent(in) :: elen,ntyp
+  character(len=elen), dimension(ntyp), intent(in) :: speciesname
+  real(kind=DP), dimension(ntyp), intent(out) :: iatomn
+  character(len=elen), allocatable :: ename_table(:)
+  integer :: it,ie,pos,ii
+  integer :: nmax_elem = 120
+
+  allocate( ename_table( nmax_elem ) );ename_table=''
+!  call set_table( 4,  1, 'H', 'He', 'Li', 'Be', 'B' )
+  ename_table(1)  = 'H'
+  ename_table(2)  = 'He'
+  ename_table(3)  = 'Li'
+  ename_table(4)  = 'Be'
+  ename_table(5)  = 'B'
+  ename_table(6)  = 'C'
+  ename_table(7)  = 'N'
+  ename_table(8)  = 'O'
+  ename_table(9)  = 'F'
+  ename_table(10) = 'Ne'
+  ename_table(11) = 'Na'
+  ename_table(12) = 'Mg'
+  ename_table(13) = 'Al'
+  ename_table(14) = 'Si'
+  ename_table(15) = 'P'
+  ename_table(16) = 'S'
+  ename_table(17) = 'Cl'
+  ename_table(18) = 'Ar'
+  ename_table(19) = 'K'
+  ename_table(20) = 'Ca'
+  ename_table(21) = 'Sc'
+  ename_table(22) = 'Ti'
+  ename_table(23) = 'V'
+  ename_table(24) = 'Cr'
+  ename_table(25) = 'Mn'
+  ename_table(26) = 'Fe'
+  ename_table(27) = 'Co'
+  ename_table(28) = 'Ni'
+  ename_table(29) = 'Cu'
+  ename_table(30) = 'Zn'
+  ename_table(31) = 'Ga'
+  ename_table(32) = 'Ge'
+  ename_table(33) = 'As'
+  ename_table(34) = 'Se'
+  ename_table(35) = 'Br'
+  ename_table(36) = 'Kr'
+  ename_table(37) = 'Rb'
+  ename_table(38) = 'Sr'
+  ename_table(39) = 'Y'
+  ename_table(40) = 'Zr'
+  ename_table(41) = 'Nb'
+  ename_table(42) = 'Mo'
+  ename_table(43) = 'Tc'
+  ename_table(44) = 'Ru'
+  ename_table(45) = 'Rh'
+  ename_table(46) = 'Pd'
+  ename_table(47) = 'Ag'
+  ename_table(48) = 'Cd'
+  ename_table(49) = 'In'
+  ename_table(50) = 'Sn'
+  ename_table(51) = 'Sb'
+  ename_table(52) = 'Te'
+  ename_table(53) = 'I'
+  ename_table(54) = 'Xe'
+  ename_table(55) = 'Cs'
+  ename_table(56) = 'Ba'
+  ename_table(57) = 'La'
+  ename_table(58) = 'Ce'
+  ename_table(59) = 'Pr'
+  ename_table(60) = 'Nd'
+  ename_table(61) = 'Pm'
+  ename_table(62) = 'Sm'
+  ename_table(63) = 'Eu'
+  ename_table(64) = 'Gd'
+  ename_table(65) = 'Tb'
+  ename_table(66) = 'Dy'
+  ename_table(67) = 'Ho'
+  ename_table(68) = 'Er'
+  ename_table(69) = 'Tm'
+  ename_table(70) = 'Yb'
+  ename_table(71) = 'Lu'
+  ename_table(72) = 'Hf'
+  ename_table(73) = 'Ta'
+  ename_table(74) = 'W'
+  ename_table(75) = 'Re'
+  ename_table(76) = 'Os'
+  ename_table(77) = 'Ir'
+  ename_table(78) = 'Pt'
+  ename_table(79) = 'Au'
+  ename_table(80) = 'Hg'
+  ename_table(81) = 'Tl'
+  ename_table(82) = 'Pb'
+  ename_table(83) = 'Bi'
+  ename_table(84) = 'Po'
+  ename_table(85) = 'At'
+  ename_table(86) = 'Rn'
+  ename_table(87) = 'Fr'
+  ename_table(88) = 'Ra'
+  ename_table(89) = 'Ac'
+  ename_table(90) = 'Th'
+  ename_table(91) = 'Pa'
+  ename_table(92) = 'U'
+  ename_table(93) = 'Np'
+  ename_table(94) = 'Pu'
+  ename_table(95) = 'Am'
+  ename_table(96) = 'Cm'
+  ename_table(97) = 'Bk'
+  ename_table(98) = 'Cf'
+  ename_table(99) = 'Es'
+  ename_table(100) = 'Fm'
+  ename_table(101) = 'Md'
+  ename_table(102) = 'No'
+  ename_table(103) = 'Lr'
+  ename_table(104) = 'Rf'
+  ename_table(105) = 'Db'
+  ename_table(106) = 'Sg'
+  ename_table(107) = 'Bh'
+  ename_table(108) = 'Hs'
+  ename_table(109) = 'Mt'
+  ename_table(110) = 'Ds'
+  ename_table(111) = 'Rg'
+  ename_table(112) = 'Cn'
+  ename_table(113) = 'Nh'
+  ename_table(114) = 'Fl'
+  ename_table(115) = 'Mc'
+  ename_table(116) = 'Lv'
+  ename_table(117) = 'Ts'
+  ename_table(118) = 'Og'
+
+  iatomn = 0 
+  do it=1,ntyp
+     do ie=1,nmax_elem 
+        if(trim(speciesname(it)) == trim(ename_table(ie))) then
+           iatomn(it) = real(ie)
+        else
+           pos = index(trim(speciesname(it)),trim(ename_table(ie)))
+           if (pos.eq.1) then
+              read(speciesname(it)(len(trim(ename_table(ie)))+1:len(speciesname(it))),*,err=90,end=90) ii
+              iatomn(it) = real(ie)
+90            continue
+           endif
+        endif
+     enddo
+  enddo
+  deallocate(ename_table) 
+
+end subroutine set_iatomn_default
+
+subroutine set_covrad_default(ntyp, iatomn, covrads)
+  use m_Const_Parameters, only : DP,BOHR
+  implicit none
+!
+  integer, intent(in) :: ntyp
+  real(kind=DP), intent(in) :: iatomn(ntyp)
+  real(kind=DP), intent(inout) :: covrads(ntyp)
+  real(kind=DP) :: ang2bohr
+  real(kind=DP), allocatable, dimension(:) :: cov_table
+!
+  integer :: it,i
+  integer :: nmax_elem = 120
+  ang2bohr = 1.d0/BOHR
+  allocate(cov_table(nmax_elem))
+
+  cov_table(:) = 1.d0
+
+  cov_table(1) = 0.32d0
+  cov_table(2) = 0.93d0
+  cov_table(3) = 1.23d0
+  cov_table(4) = 0.9d0
+  cov_table(5) = 0.82d0
+  cov_table(6) = 0.77d0
+  cov_table(7) = 0.75d0
+  cov_table(8) = 0.73d0
+  cov_table(9) = 0.72d0
+  cov_table(10) = 0.71d0
+  cov_table(11) = 1.54d0
+  cov_table(12) = 1.36d0
+  cov_table(13) = 1.18d0
+  cov_table(14) = 1.11d0
+  cov_table(15) = 1.06d0
+  cov_table(16) = 1.02d0
+  cov_table(17) = 0.99d0
+  cov_table(18) = 0.98d0
+  cov_table(19) = 2.03d0
+  cov_table(20) = 1.74d0
+  cov_table(21) = 1.44d0
+  cov_table(22) = 1.32d0
+  cov_table(23) = 1.22d0
+  cov_table(24) = 1.18d0
+  cov_table(25) = 1.17d0
+  cov_table(26) = 1.17d0
+  cov_table(27) = 1.16d0
+  cov_table(28) = 1.15d0
+  cov_table(29) = 1.17d0
+  cov_table(30) = 1.25d0
+  cov_table(31) = 1.26d0
+  cov_table(32) = 1.22d0
+  cov_table(33) = 1.2d0
+  cov_table(34) = 1.16d0
+  cov_table(35) = 1.14d0
+  cov_table(36) = 1.89d0
+  cov_table(37) = 2.16d0
+  cov_table(38) = 1.91d0
+  cov_table(39) = 1.62d0
+  cov_table(40) = 1.45d0
+  cov_table(41) = 1.34d0
+  cov_table(42) = 1.3d0
+  cov_table(43) = 1.27d0
+  cov_table(44) = 1.25d0
+  cov_table(45) = 1.25d0
+  cov_table(46) = 1.28d0
+  cov_table(47) = 1.34d0
+  cov_table(48) = 1.41d0
+  cov_table(49) = 1.44d0
+  cov_table(50) = 1.41d0
+  cov_table(51) = 1.4d0
+  cov_table(52) = 1.36d0
+  cov_table(53) = 1.33d0
+  cov_table(54) = 1.31d0
+  cov_table(55) = 2.35d0
+  cov_table(56) = 1.98d0
+  cov_table(57) = 1.25d0
+  cov_table(58) = 1.65d0
+  cov_table(59) = 1.65d0
+  cov_table(60) = 1.64d0
+  cov_table(61) = 1.63d0
+  cov_table(62) = 1.62d0
+  cov_table(63) = 1.85d0
+  cov_table(64) = 1.61d0
+  cov_table(65) = 1.59d0
+  cov_table(66) = 1.59d0
+  cov_table(67) = 1.58d0
+  cov_table(68) = 1.57d0
+  cov_table(69) = 1.56d0
+  cov_table(70) = 1.7d0
+  cov_table(71) = 1.56d0
+  cov_table(72) = 1.44d0
+  cov_table(73) = 1.34d0
+  cov_table(74) = 1.37d0
+  cov_table(75) = 1.28d0
+  cov_table(76) = 1.26d0
+  cov_table(77) = 1.27d0
+  cov_table(78) = 1.3d0
+  cov_table(79) = 1.34d0
+  cov_table(80) = 1.49d0
+  cov_table(81) = 1.48d0
+  cov_table(82) = 1.47d0
+  cov_table(83) = 1.46d0
+  cov_table(84) = 1.53d0
+  cov_table(85) = 1.47d0
+  cov_table(86) = 1.42d0
+  cov_table(87) = 2.23d0
+  cov_table(88) = 2.01d0
+  cov_table(89) = 1.86d0
+  cov_table(90) = 1.75d0
+  cov_table(91) = 1.69d0
+  cov_table(92) = 1.7d0
+  cov_table(93) = 1.71d0
+  cov_table(94) = 1.72d0
+  cov_table(95) = 1.66d0
+  cov_table(96) = 1.66d0
+  cov_table(97) = 1.66d0
+  cov_table(98) = 1.68d0
+  cov_table(99) = 1.65d0
+  cov_table(100) = 1.67d0
+  cov_table(101) = 1.73d0
+  cov_table(102) = 1.76d0
+  cov_table(103) = 1.61d0
+  cov_table(104) = 1.57d0
+  cov_table(105) = 1.49d0
+  cov_table(106) = 1.43d0
+  cov_table(107) = 1.41d0
+  cov_table(108) = 1.34d0
+  cov_table(109) = 1.29d0
+
+  do it=1,ntyp
+     i = nint( iatomn(it) )
+     covrads(it) = cov_table(i) * ang2bohr
+  enddo
+
+  deallocate(cov_table)
+
+end subroutine set_covrad_default
+
 ! === KT_add ======= 13.3B
 !*** ref. http://www.caslab.com/List-of-Elements/ *****
 !
@@ -1333,3 +1815,159 @@ contains
 
 end subroutine set_amion_default
 ! ========== 13.3B
+
+subroutine get_ac_from_dynm(nfdynm,targetframe,natm,nchar,ntyp,avec,bvec,cvec,speciesname,ityp,cps,cpd)
+  implicit none
+  integer, intent(in) :: nfdynm,targetframe,natm,nchar,ntyp
+  real(kind=8), dimension(3), intent(out) :: avec,bvec,cvec
+  character(len=nchar), dimension(ntyp), intent(out) :: speciesname
+  integer, dimension(natm), intent(out) :: ityp
+  real(kind=8), dimension(natm,3), intent(out) :: cps,cpd
+  integer, parameter :: ncharline=256
+  integer, parameter :: ncharword=20
+  integer :: nwords = 20
+  character(len=ncharline) :: line
+  character(len=ncharword), allocatable, dimension(:) :: output
+  integer :: i
+  integer :: tmpint
+  logical :: in_header, in_data
+  integer :: ntypc,currat,iframe
+  integer :: nword_out
+
+  allocate(output(nwords))
+  in_header = .false.
+  in_data = .false.
+  ntypc = 0
+  iframe = 0
+  avec=0.d0;bvec=0.d0;cvec=0.d0
+  speciesname='';ityp=0.d0;cps=0.d0
+  do 
+    read(unit=nfdynm,end=100,fmt='(a)') line
+    call split(ncharline,ncharword,nwords,line,output,nword_out)
+    if(.not. in_header .and. trim(output(1)) == '#')  then
+      in_header = .true.
+      cycle
+    endif
+    if(in_header) then
+      if(trim(output(2)) == 'a_vector')then
+        ntypc = 0
+        currat = 0
+        read(output(4),*) avec(1)
+        read(output(5),*) avec(2)
+        read(output(6),*) avec(3)
+      else if(trim(output(2)) == 'b_vector')then
+        read(output(4),*) bvec(1)
+        read(output(5),*) bvec(2)
+        read(output(6),*) bvec(3)
+      else if(trim(output(2)) == 'c_vector')then
+        read(output(4),*) cvec(1)
+        read(output(5),*) cvec(2)
+        read(output(6),*) cvec(3)
+      else if(trim(output(2)) == '(natm->type)')then
+        do i=3,nwords
+          if(len(trim(output(i)))==0) exit
+          ntypc = ntypc + 1
+          read(output(i),*) ityp(ntypc)
+        enddo
+      else if(trim(output(2)) == '(speciesname)')then
+        read(output(3),*) tmpint 
+        speciesname(tmpint)(1:nchar) = output(5)(1:nchar)
+      endif
+    endif
+    if(output(1) == 'cps')then
+      in_header = .false.
+      in_data = .true.
+      ntypc = 0
+      currat = 0
+      cycle
+    endif
+    if(in_data)then
+      currat = currat+1
+      read(output(2),*) cps(currat,1)
+      read(output(3),*) cps(currat,2)
+      read(output(4),*) cps(currat,3)
+      if(nword_out == 10)then
+        read(output(5),*) cpd(currat,1)
+        read(output(6),*) cpd(currat,2)
+        read(output(7),*) cpd(currat,3)
+      endif
+      if(currat == natm)then
+        iframe = iframe+1
+        in_data = .false.
+        if(targetframe .eq.iframe) then
+           exit
+        endif
+      endif
+    endif
+  enddo
+
+100 continue
+  deallocate(output)
+end subroutine get_ac_from_dynm
+
+subroutine get_numat_and_ntyp(nfdynm,natm,ntyp)
+  implicit none
+  integer, intent(in) :: nfdynm
+  integer, intent(out) :: natm,ntyp
+  integer, parameter :: ncharline=256
+  integer, parameter :: ncharword=20
+  integer :: nwords = 20
+  integer :: nword_out
+  character(len=ncharline) :: line
+  character(len=ncharword), allocatable, dimension(:) :: output
+  logical :: in_header
+  integer :: i
+
+  natm = 0;ntyp = 0
+  allocate(output(nwords))
+  in_header = .false.
+  do 
+    read(unit=nfdynm,end=100,fmt='(a)') line
+    call split(ncharline,ncharword,nwords,line,output,nword_out)
+    if(.not. in_header .and. trim(output(1)) == '#')  then
+      in_header = .true.
+      cycle
+    endif
+    if(in_header) then
+      if(trim(output(2)) == 'ntyp')then
+        read(output(4),*) ntyp
+        read(output(7),*) natm
+        return
+      endif
+    endif
+  enddo
+100 continue
+  write(0,*) '!** failed to resolve natm and ntyp'
+  return
+end subroutine get_numat_and_ntyp
+
+subroutine split(ncharline,ncharword,nwords,input,output,nword_out) 
+  implicit none
+  integer, intent(in) :: ncharline,ncharword,nwords
+  character(len=ncharline), intent(in) :: input
+  character(len=ncharword), dimension(nwords), intent(out) :: output
+  integer, intent(out) :: nword_out
+  integer :: i, currword,currind
+  logical :: was_ws
+  was_ws = .true.
+  currword = 0
+  currind = 0
+  do i=1,nwords
+    output(i) = ''
+  enddo
+  do i=1,ncharline
+    if (input(i:i) .eq. ' ') then
+      was_ws = .true.
+      cycle
+    endif
+    if (was_ws) then
+      currword = currword + 1
+      was_ws = .false.
+      currind = 0
+    endif
+    currind = currind+1
+    output(currword)(currind:currind) = input(i:i)
+  enddo
+  nword_out = currword
+end subroutine split
+
